@@ -1,6 +1,7 @@
 import copy
 import functools
 import importlib.util
+import json
 import logging
 import os
 import sys
@@ -12,10 +13,11 @@ from google import auth
 from octue.cloud.deployment.google.cloud_run.deployer import CloudRunDeployer
 from octue.cloud.pub_sub.service import Service
 from octue.configuration import load_service_and_app_configuration
-from octue.definitions import CHILDREN_FILENAME, FOLDER_DEFAULTS, MANIFEST_FILENAME, VALUES_FILENAME
+from octue.definitions import MANIFEST_FILENAME, VALUES_FILENAME
 from octue.log_handlers import apply_log_handler, get_remote_handler
 from octue.resources import service_backends
 from octue.runner import Runner
+from octue.utils.encoders import OctueJSONEncoder
 from twined import Twine
 
 
@@ -69,81 +71,107 @@ def octue_cli(id, logger_uri, log_level, force_reset):
 
 @octue_cli.command()
 @click.option(
-    "--app-dir",
-    type=click.Path(),
-    default=".",
-    show_default=True,
-    help="Directory containing your source code (app.py)",
-)
-@click.option(
-    "--data-dir",
-    type=click.Path(),
-    default=".",
-    show_default=True,
-    help="Location of directories containing configuration values and manifest, input values and manifest, and output "
-    "directory.",
-)
-@click.option(
-    "--config-dir",
-    type=click.Path(),
-    default=None,
-    show_default=True,
-    help="Directory containing configuration (overrides --data-dir).",
+    "-c",
+    "--service-config",
+    type=click.Path(dir_okay=False),
+    default="octue.yaml",
+    help="The path to an `octue.yaml` file defining the service to run.",
 )
 @click.option(
     "--input-dir",
-    type=click.Path(),
+    type=click.Path(file_okay=False, exists=True),
+    default=".",
+    show_default=True,
+    help="The path to a directory containing the input values (in a file called 'values.json') and/or input manifest "
+    "(in a file called 'manifest.json').",
+)
+@click.option(
+    "-o",
+    "--output-file",
+    type=click.Path(dir_okay=False),
     default=None,
     show_default=True,
-    help="Directory containing input (overrides --data-dir).",
+    help="The path to a JSON file to store the output values in.",
 )
-@click.option("--twine", type=click.Path(), default="twine.json", show_default=True, help="Location of Twine file.")
-def run(app_dir, data_dir, config_dir, input_dir, twine):
+@click.option(
+    "--output-manifest-file",
+    type=click.Path(dir_okay=False),
+    default=None,
+    help="The path to a JSON file to store the output manifest in.",
+)
+@click.option(
+    "--monitor-messages-file",
+    type=click.Path(dir_okay=False),
+    default=None,
+    show_default=True,
+    help="The path to a JSON file to store any monitor messages received in.",
+)
+def run(service_config, input_dir, output_file, output_manifest_file, monitor_messages_file):
     """Run an analysis on the given input data."""
-    config_dir = config_dir or os.path.join(data_dir, FOLDER_DEFAULTS["configuration"])
-    input_dir = input_dir or os.path.join(data_dir, FOLDER_DEFAULTS["input"])
-    twine = Twine(source=twine)
+    service_configuration, app_configuration = load_service_and_app_configuration(service_config)
 
-    (
-        configuration_values,
-        configuration_manifest,
-        input_values,
-        input_manifest,
-        children,
-    ) = set_unavailable_strand_paths_to_none(
-        twine,
-        (
-            ("configuration_values", os.path.join(config_dir, VALUES_FILENAME)),
-            ("configuration_manifest", os.path.join(config_dir, MANIFEST_FILENAME)),
-            ("input_values", os.path.join(input_dir, VALUES_FILENAME)),
-            ("input_manifest", os.path.join(input_dir, MANIFEST_FILENAME)),
-            ("children", os.path.join(config_dir, CHILDREN_FILENAME)),
-        ),
-    )
+    input_values_path = os.path.join(input_dir, VALUES_FILENAME)
+    input_manifest_path = os.path.join(input_dir, MANIFEST_FILENAME)
+
+    input_values = None
+    input_manifest = None
+
+    if os.path.exists(input_values_path):
+        input_values = input_values_path
+
+    if os.path.exists(input_manifest_path):
+        input_manifest = input_manifest_path
 
     runner = Runner(
-        app_src=app_dir,
-        twine=twine,
-        configuration_values=configuration_values,
-        configuration_manifest=configuration_manifest,
-        children=children,
+        app_src=service_configuration.app_source_path,
+        twine=Twine(source=service_configuration.twine_path),
+        configuration_values=app_configuration.configuration_values,
+        configuration_manifest=app_configuration.configuration_manifest,
+        children=app_configuration.children,
+        output_location=app_configuration.output_location,
     )
 
+    if monitor_messages_file:
+        if not os.path.exists(os.path.dirname(monitor_messages_file)):
+            os.makedirs(os.path.dirname(monitor_messages_file))
+
+        monitor_message_handler = lambda message: _add_monitor_message_to_file(monitor_messages_file, message)
+
+    else:
+        monitor_message_handler = None
+
     analysis = runner.run(
         analysis_id=global_cli_context["analysis_id"],
         input_values=input_values,
         input_manifest=input_manifest,
         analysis_log_level=global_cli_context["log_level"],
         analysis_log_handler=global_cli_context["log_handler"],
+        handle_monitor_message=monitor_message_handler,
     )
 
-    analysis.finalise()
+    click.echo(analysis.output_values)
+
+    if analysis.output_values and output_file:
+        if not os.path.exists(os.path.dirname(output_file)):
+            os.makedirs(os.path.dirname(output_file))
+
+        with open(output_file, "w") as f:
+            json.dump(analysis.output_values, f, cls=OctueJSONEncoder, indent=4)
+
+    if analysis.output_manifest:
+        if not os.path.exists(os.path.dirname(output_manifest_file)):
+            os.makedirs(os.path.dirname(output_manifest_file))
+
+        with open(output_manifest_file or f"output_manifest_{analysis.id}.json", "w") as f:
+            json.dump(analysis.output_manifest.to_primitive(), f, cls=OctueJSONEncoder, indent=4)
+
     return 0
 
 
 @octue_cli.command()
 @click.option(
-    "--service-configuration-path",
+    "-c",
+    "--service-config",
     type=click.Path(dir_okay=False),
     default="octue.yaml",
     help="The path to an `octue.yaml` file defining the service to start.",
@@ -165,9 +193,9 @@ def run(app_dir, data_dir, config_dir, input_dir, twine):
     show_default=True,
     help="Delete the Google Pub/Sub topic and subscription for the service on exit.",
 )
-def start(service_configuration_path, service_id, timeout, rm):
+def start(service_config, service_id, timeout, rm):
     """Start the service as a child to be asked questions by other services."""
-    service_configuration, app_configuration = load_service_and_app_configuration(service_configuration_path)
+    service_configuration, app_configuration = load_service_and_app_configuration(service_config)
     service_id = service_id or service_configuration.service_id
 
     runner = Runner(
@@ -213,11 +241,12 @@ def deploy():
 
 @deploy.command()
 @click.option(
-    "--octue-configuration-path",
+    "-c",
+    "--service-config",
     type=click.Path(exists=True, dir_okay=False),
     default="octue.yaml",
     show_default=True,
-    help="Path to an octue.yaml file.",
+    help="The path to an `octue.yaml` file defining the service to deploy.",
 )
 @click.option(
     "--service-id",
@@ -227,18 +256,19 @@ def deploy():
 )
 @click.option("--no-cache", is_flag=True, help="If provided, don't use the Docker cache.")
 @click.option("--update", is_flag=True, help="If provided, allow updates to an existing service.")
-def cloud_run(octue_configuration_path, service_id, update, no_cache):
+def cloud_run(service_config, service_id, update, no_cache):
     """Deploy an app as a Google Cloud Run service."""
-    CloudRunDeployer(octue_configuration_path, service_id=service_id).deploy(update=update, no_cache=no_cache)
+    CloudRunDeployer(service_config, service_id=service_id).deploy(update=update, no_cache=no_cache)
 
 
 @deploy.command()
 @click.option(
-    "--octue-configuration-path",
+    "-c",
+    "--service-config",
     type=click.Path(exists=True, dir_okay=False),
     default="octue.yaml",
     show_default=True,
-    help="Path to an octue.yaml file.",
+    help="The path to an `octue.yaml` file defining the service to deploy.",
 )
 @click.option(
     "--service-id",
@@ -254,7 +284,7 @@ def cloud_run(octue_configuration_path, service_id, update, no_cache):
     help="If provided, skip creating and running the build trigger and just deploy a pre-built image to Dataflow",
 )
 @click.option("--image-uri", type=str, default=None, help="The actual image URI to use when creating the Dataflow job.")
-def dataflow(octue_configuration_path, service_id, no_cache, update, dataflow_job_only, image_uri):
+def dataflow(service_config, service_id, no_cache, update, dataflow_job_only, image_uri):
     """Deploy an app as a Google Dataflow streaming job."""
     if bool(importlib.util.find_spec("apache_beam")):
         # Import the Dataflow deployer only if the `apache-beam` package is available (due to installing `octue` with
@@ -266,7 +296,7 @@ def dataflow(octue_configuration_path, service_id, no_cache, update, dataflow_jo
             "`pip install octue[dataflow]`"
         )
 
-    deployer = DataflowDeployer(octue_configuration_path, service_id=service_id)
+    deployer = DataflowDeployer(service_config, service_id=service_id)
 
     if dataflow_job_only:
         deployer.create_streaming_dataflow_job(image_uri=image_uri, update=update)
@@ -275,17 +305,26 @@ def dataflow(octue_configuration_path, service_id, no_cache, update, dataflow_jo
     deployer.deploy(no_cache=no_cache, update=update)
 
 
-def set_unavailable_strand_paths_to_none(twine, strands):
-    """Set paths to unavailable strands to None, leaving the paths of available strands as they are."""
-    updated_strand_paths = []
+def _add_monitor_message_to_file(path, monitor_message):
+    """Add a monitor message to the file at the given path.
 
-    for strand_name, strand in strands:
-        if strand_name not in twine.available_strands:
-            updated_strand_paths.append(None)
-        else:
-            updated_strand_paths.append(strand)
+    :param str path: the path of the file to add the monitor message to
+    :param dict monitor_message: the monitor message to add to the file
+    :return None:
+    """
+    previous_messages = []
 
-    return updated_strand_paths
+    if os.path.exists(path):
+        try:
+            with open(path) as f:
+                previous_messages = json.load(f)
+        except json.decoder.JSONDecodeError:
+            pass
+
+    previous_messages.append(monitor_message)
+
+    with open(path, "w") as f:
+        json.dump(previous_messages, f)
 
 
 if __name__ == "__main__":
