@@ -12,14 +12,13 @@ import requests
 from octue.cloud import storage
 from octue.cloud.storage import GoogleCloudStorageClient
 from octue.exceptions import CloudLocationNotSpecified, InvalidInputException
-from octue.migrations.cloud_storage import translate_bucket_name_and_path_in_bucket_to_cloud_path
-from octue.mixins import Hashable, Identifiable, Labelable, Metadata, Serialisable, Taggable
+from octue.mixins import CloudPathable, Hashable, Identifiable, Labelable, Metadata, Serialisable, Taggable
 from octue.resources.datafile import Datafile
 from octue.resources.filter_containers import FilterSet
 from octue.resources.label import LabelSet
 from octue.resources.tag import TagDict
 from octue.utils.encoders import OctueJSONEncoder
-from octue.utils.metadata import METADATA_FILENAME, load_local_metadata_file
+from octue.utils.metadata import METADATA_FILENAME, load_local_metadata_file, overwrite_local_metadata_file
 
 
 logger = logging.getLogger(__name__)
@@ -28,19 +27,19 @@ logger = logging.getLogger(__name__)
 SIGNED_METADATA_DIRECTORY = ".signed_metadata_files"
 
 
-class Dataset(Labelable, Taggable, Serialisable, Identifiable, Hashable, Metadata):
-    """A representation of a dataset, containing files, labels, etc.
+class Dataset(Labelable, Taggable, Serialisable, Identifiable, Hashable, Metadata, CloudPathable):
+    """A representation of a dataset. The default usage is to provide the path to a local or cloud directory and create
+    the dataset from the files it contains. Alternatively, the `files` parameter can be provided and only those files
+    are included. Either way, the `path` parameter should be explicitly set to something meaningful.
 
-    This is used to read a list of files (and their associated properties) into octue analysis, or to compile a
-    list of output files (results) and their properties that will be sent back to the octue system.
-
-    :param iter(dict|octue.resources.datafile.Datafile) files: the files belonging to the dataset
-    :param str|None name:
-    :param str|None id:
-    :param str|None path:
-    :param dict|octue.resources.tag.TagDict|None tags:
-    :param iter(str)|octue.resources.label.LabelSet|None labels:
+    :param str|None path: the path to the dataset (defaults to the current working directory if none is given)
+    :param iter(str|dict|octue.resources.datafile.Datafile)|None files: the files belonging to the dataset
+    :param bool recursive: if `True`, include in the dataset all files in the subdirectories recursively contained within the dataset directory
     :param bool hypothetical: if `True`, ignore any metadata stored for this dataset locally or in the cloud and use whatever is given at instantiation
+    :param str|None id: an optional UUID to assign to the dataset (defaults to a random UUID if none is given)
+    :param str|None name: an optional name to give to the dataset (defaults to the dataset directory name)
+    :param dict|octue.resources.tag.TagDict|None tags: key-value pairs with string keys conforming to the Octue tag format (see `TagDict`)
+    :param iter(str)|octue.resources.label.LabelSet|None labels: space-separated string of labels relevant to the dataset
     :return None:
     """
 
@@ -50,89 +49,51 @@ class Dataset(Labelable, Taggable, Serialisable, Identifiable, Hashable, Metadat
     # Paths to files are added to the serialisation in `Dataset.to_primitive`.
     _SERIALISE_FIELDS = (*_METADATA_ATTRIBUTES, "path")
 
-    def __init__(self, files=None, name=None, id=None, path=None, tags=None, labels=None):
-        super().__init__(name=name, id=id, tags=tags, labels=labels)
-        self.path = path or os.getcwd()
-        self.files = self._instantiate_datafiles(files or [])
-
-    @classmethod
-    def from_local_directory(cls, path_to_directory, recursive=False, hypothetical=False, **kwargs):
-        """Instantiate a Dataset from the files in the given local directory. If a dataset metadata file is present,
-        that is used to decide which files are in the dataset.
-
-        :param str path_to_directory: path to a local directory
-        :param bool recursive: if `True`, include all files in the directory's subdirectories recursively
-        :param bool hypothetical: if `True`, don't use any metadata stored for this dataset locally
-        :param kwargs: other keyword arguments for the `Dataset` instantiation
-        :return Dataset:
-        """
-        datafiles = FilterSet()
-
-        for level, (directory_path, _, filenames) in enumerate(os.walk(path_to_directory)):
-            for filename in filenames:
-
-                if filename == METADATA_FILENAME:
-                    continue
-
-                if not recursive and level > 0:
-                    break
-
-                datafiles.add(Datafile(path=os.path.join(directory_path, filename)))
-
-        dataset = Dataset(path=path_to_directory, files=datafiles, **kwargs)
-
-        if not hypothetical:
-            dataset._use_local_metadata()
-
-        dataset._warn_about_metadata_override(hypothetical=hypothetical, **kwargs)
-        return dataset
-
-    @classmethod
-    def from_cloud(
-        cls,
-        cloud_path=None,
-        bucket_name=None,
-        path_to_dataset_directory=None,
+    def __init__(
+        self,
+        path=None,
+        files=None,
         recursive=False,
         hypothetical=False,
-        **kwargs,
+        id=None,
+        name=None,
+        tags=None,
+        labels=None,
     ):
-        """Instantiate a Dataset from Google Cloud storage. The dataset's files are collected by scanning its cloud
-        directory unless a "files" key is present in the dataset metadata, in which case the files specified there are
-        used.
+        super().__init__(name=name, id=id, tags=tags, labels=labels)
+        self.path = path or os.getcwd()
+        self.files = FilterSet()
 
-        :param str|None cloud_path: full path to dataset directory in cloud storage (e.g. `gs://bucket_name/path/to/dataset`)
-        :param bool recursive: if `True`, include in the dataset all files in the subdirectories recursively contained in the dataset directory
-        :param bool hypothetical: if `True`, don't use any metadata stored for this dataset in the cloud
-        :param kwargs: other keyword arguments for the `Dataset` instantiation
-        :return Dataset:
-        """
-        if not cloud_path:
-            cloud_path = translate_bucket_name_and_path_in_bucket_to_cloud_path(bucket_name, path_to_dataset_directory)
-
-        bucket_name = storage.path.split_bucket_name_from_cloud_path(cloud_path)[0]
-
-        dataset = Dataset(path=cloud_path, **kwargs)
-
-        if not hypothetical:
-            dataset._use_cloud_metadata()
-
-        if not dataset.files:
-            dataset.files = FilterSet(
-                Datafile(path=storage.path.generate_gs_path(bucket_name, blob.name))
-                for blob in GoogleCloudStorageClient().scandir(
-                    cloud_path,
-                    recursive=recursive,
-                    filter=(
-                        lambda blob: (
-                            not blob.name.endswith(METADATA_FILENAME) and SIGNED_METADATA_DIRECTORY not in blob.name
-                        )
-                    ),
+        if files:
+            if not any((isinstance(files, list), isinstance(files, set), isinstance(files, tuple))):
+                raise InvalidInputException(
+                    "The `files` parameter of a `Dataset` must be an iterable of `Datafile` instances, strings, or "
+                    f"dictionaries. Received {files!r} instead."
                 )
-            )
 
-        dataset._warn_about_metadata_override(hypothetical=hypothetical, **kwargs)
-        return dataset
+            self.files = self._instantiate_datafiles(files)
+            return
+
+        if storage.path.is_cloud_path(self.path):
+            self._instantiate_from_cloud(path=self.path, recursive=recursive, hypothetical=hypothetical)
+        else:
+            self._instantiate_from_local_directory(path=self.path, recursive=recursive, hypothetical=hypothetical)
+
+        if hypothetical:
+            logger.debug("Ignored stored metadata for %r.", self)
+            return
+
+        if self.metadata(include_sdk_version=False) != {
+            "name": name or self.name,
+            "id": id or self.id,
+            "tags": TagDict(tags),
+            "labels": LabelSet(labels),
+        }:
+            logger.warning(
+                "Overriding metadata given at instantiation with stored metadata for %r - set `hypothetical` to `True` "
+                "at instantiation to avoid this.",
+                self,
+            )
 
     @property
     def name(self):
@@ -157,14 +118,6 @@ class Dataset(Labelable, Taggable, Serialisable, Identifiable, Hashable, Metadat
         """
         self._name = name
 
-    @property
-    def exists_in_cloud(self):
-        """Return `True` if the dataset exists in the cloud.
-
-        :return bool:
-        """
-        return storage.path.is_cloud_path(self.path)
-
     @property
     def exists_locally(self):
         """Return `True` if the dataset exists locally.
@@ -173,26 +126,6 @@ class Dataset(Labelable, Taggable, Serialisable, Identifiable, Hashable, Metadat
         """
         return not self.exists_in_cloud
 
-    @property
-    def bucket_name(self):
-        """Get the name of the bucket the dataset exists in if it exists in the cloud.
-
-        :return str|None:
-        """
-        if self.exists_in_cloud:
-            return storage.path.split_bucket_name_from_cloud_path(self.path)[0]
-        return None
-
-    @property
-    def path_in_bucket(self):
-        """Get the path of the dataset in its bucket if it exists in the cloud.
-
-        :return str|None:
-        """
-        if self.exists_in_cloud:
-            return storage.path.split_bucket_name_from_cloud_path(self.path)[1]
-        return None
-
     @property
     def all_files_are_in_cloud(self):
         """Do all the files of the dataset exist in the cloud?
@@ -238,19 +171,23 @@ class Dataset(Labelable, Taggable, Serialisable, Identifiable, Hashable, Metadat
         """
         self.update_metadata()
 
-    def to_cloud(self, cloud_path=None, bucket_name=None, output_directory=None):
+    def upload(self, cloud_path=None):
         """Upload a dataset to the given cloud path.
 
         :param str|None cloud_path: cloud path to store dataset at (e.g. `gs://bucket_name/path/to/dataset`)
         :return str: cloud path for dataset
         """
-        if not cloud_path:
-            cloud_path = translate_bucket_name_and_path_in_bucket_to_cloud_path(bucket_name, output_directory)
+        cloud_path = self._get_cloud_location(cloud_path)
 
         files_and_paths = []
 
         for datafile in self.files:
-            datafile_path_relative_to_dataset = self._datafile_path_relative_to_self(datafile, path_type="local_path")
+            if self.exists_in_cloud:
+                path_type = "cloud_path"
+            else:
+                path_type = "local_path"
+
+            datafile_path_relative_to_dataset = self._datafile_path_relative_to_self(datafile, path_type=path_type)
 
             files_and_paths.append(
                 (
@@ -259,7 +196,7 @@ class Dataset(Labelable, Taggable, Serialisable, Identifiable, Hashable, Metadat
                 )
             )
 
-        def upload(iterable_element):
+        def upload_datafile(iterable_element):
             """Upload a datafile to the given cloud path.
 
             :param tuple(octue.resources.datafile.Datafile, str) iterable_element:
@@ -267,12 +204,12 @@ class Dataset(Labelable, Taggable, Serialisable, Identifiable, Hashable, Metadat
             """
             datafile = iterable_element[0]
             cloud_path = iterable_element[1]
-            datafile.to_cloud(cloud_path=cloud_path)
+            datafile.upload(cloud_path=cloud_path)
             return datafile.cloud_path
 
         # Use multiple threads to significantly speed up file uploads by reducing latency.
         with concurrent.futures.ThreadPoolExecutor() as executor:
-            for path in executor.map(upload, files_and_paths):
+            for path in executor.map(upload_datafile, files_and_paths):
                 logger.debug("Uploaded datafile to %r.", path)
 
         self.path = cloud_path
@@ -311,9 +248,7 @@ class Dataset(Labelable, Taggable, Serialisable, Identifiable, Hashable, Metadat
         existing_metadata_records = load_local_metadata_file(self._metadata_path)
         existing_metadata_records["dataset"] = self.to_primitive(include_files=False)
         os.makedirs(self.path, exist_ok=True)
-
-        with open(self._metadata_path, "w") as f:
-            json.dump(existing_metadata_records, f, cls=OctueJSONEncoder)
+        overwrite_local_metadata_file(data=existing_metadata_records, path=self._metadata_path)
 
     def generate_signed_url(self, expiration=datetime.timedelta(days=7)):
         """Generate a signed URL for the dataset. This is done by uploading a uniquely named metadata file containing
@@ -360,13 +295,13 @@ class Dataset(Labelable, Taggable, Serialisable, Identifiable, Hashable, Metadat
             if datafile.exists_in_cloud:
 
                 if datafile.cloud_path != new_cloud_path and not datafile.cloud_path.startswith(self.path):
-                    datafile.to_cloud(new_cloud_path)
+                    datafile.upload(new_cloud_path)
 
                 self.files.add(datafile)
                 return
 
             # Add a local datafile to a cloud dataset.
-            datafile.to_cloud(new_cloud_path)
+            datafile.upload(new_cloud_path)
             self.files.add(datafile)
             return
 
@@ -393,7 +328,7 @@ class Dataset(Labelable, Taggable, Serialisable, Identifiable, Hashable, Metadat
         """
         return self.files.one(labels__contains=label)
 
-    def download_all_files(self, local_directory=None):
+    def download(self, local_directory=None):
         """Download all files in the dataset into the given local directory. If no path to a local directory is given,
         the files will be downloaded to temporary locations.
 
@@ -419,7 +354,7 @@ class Dataset(Labelable, Taggable, Serialisable, Identifiable, Hashable, Metadat
             local_path = os.path.abspath(os.path.join(local_directory, *path_relative_to_dataset.split("/")))
             files_and_paths.append((file, local_path))
 
-        def download(iterable_element):
+        def download_datafile(iterable_element):
             """Download a datafile to the given path.
 
             :param tuple(octue.resources.datafile.Datafile, str) iterable_element:
@@ -431,10 +366,10 @@ class Dataset(Labelable, Taggable, Serialisable, Identifiable, Hashable, Metadat
 
         # Use multiple threads to significantly speed up files downloads by reducing latency.
         with concurrent.futures.ThreadPoolExecutor() as executor:
-            for path in executor.map(download, files_and_paths):
+            for path in executor.map(download_datafile, files_and_paths):
                 logger.debug("Downloaded datafile to %r.", path)
 
-        logger.info("Downloaded %r dataset to %r.", self.name, local_directory)
+        logger.info("Downloaded %r to %r.", self, local_directory)
 
     def to_primitive(self, include_files=True):
         """Convert the dataset to a dictionary of primitives, converting its files into their paths for a lightweight
@@ -455,6 +390,57 @@ class Dataset(Labelable, Taggable, Serialisable, Identifiable, Hashable, Metadat
 
         return serialised_dataset
 
+    def _instantiate_from_cloud(self, path, recursive=True, hypothetical=False):
+        """Instantiate the dataset from a cloud directory.
+
+        :param str path: the cloud path to a directory in cloud storage
+        :param bool recursive: if `True`, include in the dataset all files in the subdirectories recursively contained within the dataset directory
+        :param bool hypothetical: if `True`, ignore any metadata stored for this dataset in the cloud and use whatever is given at instantiation
+        :return None:
+        """
+        if not hypothetical:
+            self._use_cloud_metadata()
+
+        if not self.files:
+            bucket_name = storage.path.split_bucket_name_from_cloud_path(path)[0]
+
+            self.files = FilterSet(
+                Datafile(path=storage.path.generate_gs_path(bucket_name, blob.name))
+                for blob in GoogleCloudStorageClient().scandir(
+                    path,
+                    recursive=recursive,
+                    filter=(
+                        lambda blob: (
+                            not blob.name.endswith(METADATA_FILENAME) and SIGNED_METADATA_DIRECTORY not in blob.name
+                        )
+                    ),
+                )
+            )
+
+    def _instantiate_from_local_directory(self, path, recursive=False, hypothetical=False):
+        """Instantiate the dataset from a local directory.
+
+        :param str path: the path to a local directory
+        :param bool recursive: if `True`, include in the dataset all files in the subdirectories recursively contained within the dataset directory
+        :param bool hypothetical: if `True`, ignore any metadata stored for this dataset locally and use whatever is given at instantiation
+        :return None:
+        """
+        self.files = FilterSet()
+
+        for level, (directory_path, _, filenames) in enumerate(os.walk(path)):
+            for filename in filenames:
+
+                if filename == METADATA_FILENAME:
+                    continue
+
+                if not recursive and level > 0:
+                    break
+
+                self.files.add(Datafile(path=os.path.join(directory_path, filename)))
+
+        if not hypothetical:
+            self._use_local_metadata()
+
     def _instantiate_datafiles(self, files):
         """Instantiate and add the given files to a `FilterSet`.
 
@@ -561,27 +547,3 @@ class Dataset(Labelable, Taggable, Serialisable, Identifiable, Hashable, Metadat
             datafile_path = datafile_path.split("?")[0]
 
         return storage.path.relpath(datafile_path, dataset_path)
-
-    def _warn_about_metadata_override(self, hypothetical, **kwargs):
-        """Issue a warning about instantiation metadata override if `hypothetical` is `False` and the dataset's metadata
-        is different from the provided instantiation keyword arguments.
-
-        :param bool hypothetical: if `True`, don't raise any warnings.
-        :param kwargs: the Dataset instantiation keyword arguments
-        :return None:
-        """
-        if hypothetical:
-            logger.debug("Ignored stored metadata for %r.", self)
-            return
-
-        if self.metadata(include_sdk_version=False) != {
-            "name": kwargs.get("name") or self.name,
-            "id": kwargs.get("id") or self.id,
-            "tags": TagDict(kwargs.get("tags")),
-            "labels": LabelSet(kwargs.get("labels")),
-        }:
-            logger.warning(
-                "Overriding metadata given at instantiation with stored metadata for %r - set `hypothetical` to `True` "
-                "at instantiation to avoid this.",
-                self,
-            )
